#include "3d.h"

point2d_t tr3d2d(point3d_t p3d){
  point2d_t p2d;
  p2d.x = p3d.x * R / p3d.z + X0;
  p2d.y = p3d.y * R / p3d.z + Y0;
  return p2d;
}

point2d_t tr3d2d_org(point3d_t p3d, int x0, int y0){
  point2d_t p2d;
  p2d.x = p3d.x * R / p3d.z + x0;
  p2d.y = p3d.y * R / p3d.z + y0;
  return p2d;
}

point3d_t add_vector3d(point3d_t pos1, point3d_t pos2){
  point3d_t ret;
  ret.x = pos1.x + pos2.x;
  ret.y = pos1.y + pos2.y;
  ret.z = pos1.z + pos2.z;
  return ret;
}


point3d_t reverse_vector3d(point3d_t pos){
  point3d_t ret;
  ret.x = -pos.x;
  ret.y = -pos.y;
  ret.z = -pos.z;
  return ret;
}

point3d_t rot_zy(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.z = (int)(p3d.z * cos(r) - p3d.y * sin(r));
  new_p3d.y = (int)(p3d.z * sin(r) + p3d.y * cos(r));
  new_p3d.x = p3d.x;
  return new_p3d;
}

point3d_t rot_xy(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.x = (int)(p3d.x * cos(r) - p3d.y * sin(r));
  new_p3d.y = (int)(p3d.x * sin(r) + p3d.y * cos(r));
  new_p3d.z = p3d.z;
  return new_p3d;
}

point3d_t rot_xz(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.x = (int)(p3d.x * cos(r) - p3d.z * sin(r));
  new_p3d.z = (int)(p3d.x * sin(r) + p3d.z * cos(r));
  new_p3d.y = p3d.y;
  return new_p3d;
}

point3d_t resize(point3d_t p3d, double scale){
  p3d.x = (int)(p3d.x * scale);
  p3d.y = (int)(p3d.y * scale);
  p3d.z = (int)(p3d.z * scale);
  return p3d;
}

void resize_line(point3d_t *st, point3d_t *en, double scale){
    point3d_t mid;
    mid.x = (st->x + en->x) / 2;
    mid.y = (st->x + en->y) / 2;
    mid.z = (st->x + en->z) / 2;
    st->x = (int)(mid.x + (st->x - mid.x) * scale);
    st->y = (int)(mid.y + (st->y - mid.y) * scale);
    st->z = (int)(mid.z + (st->z - mid.z) * scale);
    en->x = (int)(mid.x + (en->x - mid.x) * scale);
    en->y = (int)(mid.y + (en->y - mid.y) * scale);
    en->z = (int)(mid.z + (en->z - mid.z) * scale);
}
